@@ -26,10 +26,12 @@
 
 #include "extern.h"
 #include "watch_err.h"
+#include "gettime.h"
 
 static int watchdog_fd = -1;
 static int timeout_used = TIMER_MARGIN;
 static int Refresh_using_ioctl = FALSE;
+static struct timespec tlast = {0, 0};
 
 /*
  * Open the watchdog timer (if name non-NULL) and set the time-out value (if non-zero).
@@ -65,6 +67,9 @@ int open_watchdog(char *name, int timeout)
 		}
 	}
 
+	/* Start timer for minimum 'ping' on device open. */
+	clock_gettime(CLOCK_MONOTONIC, &tlast);
+
 	/* The IT8728 on Gigabyte motherboard (and similar) would trip due to the normal
 	 * refresh in the device driver failing to reset the timer for no obvious reason
 	 * (though the normal operation used the Consumer IR sender to refresh via an
@@ -72,12 +77,35 @@ int open_watchdog(char *name, int timeout)
 	 * time-out every refresh operation.
 	 *
 	 * See https://bugs.launchpad.net/ubuntu/+source/linux/+bug/932381
+	 * Also https://bugzilla.kernel.org/show_bug.cgi?id=42801
 	 *
 	 */
+
 	Refresh_using_ioctl = FALSE;
-	if (strcmp("IT87 WDT", (char *)ident.identity) == 0) {
-		Refresh_using_ioctl = TRUE;
-		log_message(LOG_INFO, "Running IT87 module fix-up");
+
+	switch (refresh_use_settimeout) {
+		case ENUM_NO:
+			/* Set to "no" so never use ioctl mode. */
+			break;
+
+		case ENUM_YES:
+			/* Set to "yes" so always use ioctl mode. */
+			Refresh_using_ioctl = TRUE;
+			log_message(LOG_INFO, "Running ioctl-based refresh");
+			break;
+
+		case ENUM_AUTO:
+			/* Set to "auto" to decide based on driver identity. */
+			Refresh_using_ioctl = FALSE;
+			if (strcmp("IT87 WDT", (char *)ident.identity) == 0) {
+				Refresh_using_ioctl = TRUE;
+				log_message(LOG_INFO, "Running IT87 module fix-up");
+			}
+			break;
+
+		default:
+			log_message(LOG_ERR, "Unknown ioctl selection mode (%d)", refresh_use_settimeout);
+			break;
 	}
 
 	return rv;
@@ -93,7 +121,7 @@ int open_watchdog(char *name, int timeout)
 static int try_other_times(const int timeout)
 {
 	static const int try_values[] = {3, 5, 10};
-	static const int num_try = sizeof(try_values) / sizeof(int);
+	static const int num_try = ARRAY_SIZE(try_values);
 	int ii;
 
 	for (ii = 0; ii < num_try; ii++) {
@@ -155,14 +183,53 @@ int set_watchdog_timeout(int timeout)
 	return rv;
 }
 
+/*
+ * Test to see if "a - b > td" for a time-out indication.
+ *
+ * Return zero if a-b is between 0 and td.
+ *
+ * Return non-zero if a-b is negative (clock stepped?) or greater than td
+ */
+
+static int timespecpast(const struct timespec *a, const struct timespec *b, const struct timespec *td)
+{
+	struct timespec tdiff;
+	int ret = 0;
+
+	/* Compute "a - b" */
+	timespecsub(a, b, &tdiff);
+
+	if (tdiff.tv_sec < 0 || timespeccmp(&tdiff, td, >)) {
+		ret = 1;
+	}
+
+	if (verbose > 1) {
+		log_message(LOG_DEBUG, "Time from last refresh = %ld.%09ld => %d", tdiff.tv_sec, tdiff.tv_nsec, ret);
+	}
+
+	return ret;
+}
+
 /* write to the watchdog device */
 int keep_alive(void)
 {
 	int err = ENOERR;
+	struct timespec tnow;
+	const struct timespec tminimum = {0, NSEC/5}; /* Set to 0.2 seconds minimum 'ping' time. */
 
 	if (watchdog_fd == -1)
 		return (ENOERR);
 
+	/* Check if we have passed minimum period. */
+	clock_gettime(CLOCK_MONOTONIC, &tnow);
+
+	if (timespecpast(&tnow, &tlast, &tminimum) == 0) {
+		return (ENOERR);
+	}
+
+	/* Once we are going to feed the dog, save this time for next check. */
+	tlast = tnow;
+
 	if (Refresh_using_ioctl) {
 		int timeout = timeout_used;
 		if (ioctl(watchdog_fd, WDIOC_SETTIMEOUT, &timeout) < 0) {
@@ -177,6 +244,15 @@ int keep_alive(void)
 		}
 	}
 
+	/*
+	 * If we set this option then we simply ignore any errors reported by writing to
+	 * the watchdog device. Typically for broken IPMI implementations such as:
+	 * https://support.gfi.com/hc/en-us/articles/360012894154-IPMI-Watchdog-Response-Error
+	 */
+	if (refresh_ignore_errors) {
+		err = ENOERR;
+	}
+
 	/* MJ 20/2/2001 write a heartbeat to a file outside the syslog, because:
 	   - there is no guarantee the system logger is up and running
 	   - easier and quicker to parse checkpoint information */
