@@ -2,10 +2,6 @@
  *
  * Code for checking the system load averages.
  *
- * TO DO:
- *  The 'repair' option possible in the watchdog call must fail as these are averages and
- *  so take time to drop. Really need some timer before deciding a reboot is the best option.
- *  For the time being this timer has to be implemented in the repair script.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,7 +27,9 @@ int open_loadcheck(void)
 {
 	int rv = -1;
 
-	if (maxload1 > 0) {
+	close_loadcheck();
+
+	if (maxload1 || maxload5 || maxload15) {
 		/* open the load average file */
 		load_fd = open(load_name, O_RDONLY);
 		if (load_fd == -1) {
@@ -50,32 +48,27 @@ int check_load(void)
 {
 	int avg1, avg5, avg15;
 	char buf[40], *ptr;
+	int n;
 
 	/* is the load average file open? */
-	if (load_fd == -1 || maxload1 == 0 || maxload5 == 0 || maxload15 == 0)
+	if (load_fd == -1)
 		return (ENOERR);
 
 	/* position pointer at start of file */
 	if (lseek(load_fd, 0, SEEK_SET) < 0) {
 		int err = errno;
 		log_message(LOG_ERR, "lseek %s gave errno = %d = '%s'", load_name, err, strerror(err));
-
-		if (softboot)
-			return (err);
-
-		return (ENOERR);
+		return (err);
 	}
 
 	/* read the line (there is only one) */
-	if (read(load_fd, buf, sizeof(buf)) < 0) {
+	if ((n = read(load_fd, buf, sizeof(buf)-1)) < 0) {
 		int err = errno;
 		log_message(LOG_ERR, "read %s gave errno = %d = '%s'", load_name, err, strerror(err));
-
-		if (softboot)
-			return (err);
-
-		return (ENOERR);
+		return (err);
 	}
+	/* Force string to be nul-terminated. */
+	buf[n] = 0;
 	/* we only care about integer values */
 	avg1 = atoi(buf);
 
@@ -86,21 +79,20 @@ int check_load(void)
 		avg5 = atoi(ptr);
 		ptr = strchr(ptr + 1, ' ');
 	}
-	if (ptr != NULL)
+
+	if (ptr != NULL) {
 		avg15 = atoi(ptr);
-	else {
+	} else {
 		log_message(LOG_ERR, "%s does not contain any data (read = %s)", load_name, buf);
-
-		if (softboot)
-			return (ENOLOAD);
-
-		return (ENOERR);
+		return (ENOLOAD);
 	}
 
 	if (verbose && logtick && ticker == 1)
-		log_message(LOG_INFO, "current load is %d %d %d", avg1, avg5, avg15);
+		log_message(LOG_DEBUG, "current load is %d %d %d", avg1, avg5, avg15);
 
-	if (avg1 > maxload1 || avg5 > maxload5 || avg15 > maxload15) {
+	if ((maxload1  > 0 && avg1  > maxload1) ||
+		(maxload5  > 0 && avg5  > maxload5) ||
+		(maxload15 > 0 && avg15 > maxload15)) {
 
 		log_message(LOG_ERR, "loadavg %d %d %d is higher than the given threshold %d %d %d!",
 							avg1, avg5, avg15,
