@@ -10,7 +10,7 @@
 #endif
 
 #include <errno.h>
-#include <sys/time.h>
+#include <time.h>
 #include <netinet/ip.h>
 #include <linux/icmp.h>
 #include <fcntl.h>
@@ -31,8 +31,7 @@
 
 #include "extern.h"
 #include "watch_err.h"
-
-static const long USEC = 1000000L;
+#include "gettime.h"
 
 /*
  * in_cksum --
@@ -62,6 +61,93 @@ static int in_cksum(unsigned short *addr, int len)
 	return (answer);
 }
 
+/*
+ * Send out a ping packet of sequence count 'i' and ID from 'daemon_pid' value.
+ *
+ * Return value is the same as 'ecode' and non-zero on error case.
+ */
+static int send_ping(char *target, int sock_fp, struct sockaddr to, int i, int *ecode)
+{
+	unsigned char outpack[MAXPACKET];
+	memset(outpack, 0, sizeof(outpack));
+	struct icmphdr *icp = (struct icmphdr *)outpack;
+	int err = ENOERR;
+
+	/* setup a ping message */
+	icp->type = ICMP_ECHO;
+	icp->code = icp->checksum = 0;
+	icp->un.echo.sequence = htons(i + 1);
+	icp->un.echo.id = htons(daemon_pid);	/* ID */
+
+	/* compute ICMP checksum here */
+	icp->checksum = in_cksum((unsigned short *)icp, DATALEN + 8);
+
+	/* and send it out */
+	if (sendto(sock_fp, (char *)outpack, DATALEN + 8, 0, &to, sizeof(struct sockaddr)) < 0) {
+		err = errno;
+
+		/* if our kernel tells us the network is unreachable we are done */
+		if (err == ENETUNREACH) {
+			log_message(LOG_ERR, "network is unreachable (target: %s)", target);
+		} else {
+			log_message(LOG_ERR, "sendto gave error for target %s = %d = '%s'", target, err, strerror(err));
+		}
+	}
+
+	*ecode = err;
+
+return err;
+}
+
+/*
+ * Look for a ping reply. We check it is our ping by comparing the ID and sequence
+ * count to see if they match.
+ *
+ * Return value is non-zero on something significant: either an error or finding
+ * one of our ping's response. The 'ecode' value shows which it was (0 if good).
+ *
+ */
+static int found_ping(unsigned char *packet, int sock_fp, struct sockaddr to, int i,
+					  int *ecode, fd_set *fdmask, struct timespec *dtimeout)
+{
+	struct sockaddr_in *to_in = (struct sockaddr_in *)&to;
+	struct sockaddr_in from;
+	socklen_t fromlen;
+
+	if (pselect(sock_fp + 1, fdmask, NULL, NULL, dtimeout, NULL) >= 1) {
+		/* read reply */
+		fromlen = sizeof(from);
+		if (recvfrom(sock_fp, packet, PKBUF_SIZE, 0, (struct sockaddr *)&from, &fromlen) < 0) {
+			int err = errno;
+
+			if (err != EINTR) {
+				log_message(LOG_ERR, "recvfrom gave errno = %d = '%s'", err, strerror(err));
+				*ecode = err;
+				return 1;
+			}
+		} else {
+			/* check if packet is our ECHO */
+			struct icmphdr *icp = (struct icmphdr *)(packet + (((struct ip *)packet)->ip_hl << 2));
+
+			if (icp->type == ICMP_ECHOREPLY) {
+				int rcv_id  = ntohs(icp->un.echo.id);
+				int rcv_seq = ntohs(icp->un.echo.sequence);
+
+				/* Have ping reply, but is it the one we just sent? */
+				if (rcv_id  == daemon_pid &&
+					rcv_seq == (i + 1) &&
+					from.sin_addr.s_addr == to_in->sin_addr.s_addr) {
+
+					*ecode = ENOERR;
+					return 1;
+				}
+			}
+		}
+	}
+
+	return 0;
+}
+
 /*
  * Check network / machine is accessible via 'ping' packet.
  */
@@ -69,8 +155,8 @@ static int in_cksum(unsigned short *addr, int len)
 int check_net(char *target, int sock_fp, struct sockaddr to, unsigned char *packet, int time, int count)
 {
 	int i;
-	unsigned char outpack[MAXPACKET];
-	struct timeval tmax;
+	int err = 0;
+	struct timespec tmax;
 	ldiv_t d;
 
 	if (target == NULL)
@@ -79,103 +165,46 @@ int check_net(char *target, int sock_fp, struct sockaddr to, unsigned char *pack
 	if (count < 1)
 		return (EINVAL);
 
-	memset(outpack, 0, sizeof(outpack));
-
 	/* set the timeout value */
 	d = ldiv(time, count);
 	tmax.tv_sec = d.quot;
-	/* Compute microseconds, including the above remainder. */
-	tmax.tv_usec = (d.rem * USEC) / count;
+	/* Compute nanoseconds, including the above remainder. */
+	tmax.tv_nsec = (d.rem * NSEC) / count;
 
 	/* try "ping-count" times */
 	for (i = 0; i < count; i++) {
-
-		struct sockaddr_in from;
 		fd_set fdmask;
-		socklen_t fromlen;
-		struct timeval tstart, timeout, dtimeout;
-		struct icmphdr *icp = (struct icmphdr *)outpack;
-		struct sockaddr_in *to_in = (struct sockaddr_in *)&to;
+		struct timespec tstart, timeout, dtimeout;
 
-		/* setup a ping message */
-		icp->type = ICMP_ECHO;
-		icp->code = icp->checksum = 0;
-		icp->un.echo.sequence = htons(i + 1);
-		icp->un.echo.id = htons(daemon_pid);	/* ID */
-
-		/* compute ICMP checksum here */
-		icp->checksum = in_cksum((unsigned short *)icp, DATALEN + 8);
-
-		/* and send it out */
-		if (sendto(sock_fp, (char *)outpack, DATALEN + 8, 0, &to, sizeof(struct sockaddr)) < 0) {
-			int err = errno;
-
-			/* if our kernel tells us the network is unreachable we are done */
-			if (err == ENETUNREACH) {
-				log_message(LOG_ERR, "network is unreachable (target: %s)", target);
-			} else {
-				log_message(LOG_ERR, "sendto gave error for target %s = %d = '%s'", target, err, strerror(err));
-			}
-
-			return (err);
+		if (send_ping(target, sock_fp, to, i, &err)) {
+			return err;
 		}
 
-		gettimeofday(&tstart, NULL);
+		clock_gettime(CLOCK_MONOTONIC, &tstart);
 		/* set the timeout value */
-		timeradd(&tstart, &tmax, &timeout);
+		timespecadd(&tstart, &tmax, &timeout);
 
 		/* wait for reply */
 		FD_ZERO(&fdmask);
 		FD_SET(sock_fp, &fdmask);
 		while (1) {
-			gettimeofday(&dtimeout, NULL);
-			timersub(&timeout, &dtimeout, &dtimeout);
+			clock_gettime(CLOCK_MONOTONIC, &dtimeout);
+			timespecsub(&timeout, &dtimeout, &dtimeout);
 			/* Check if we have timed out waiting for a reply. */
 			if ((long)dtimeout.tv_sec < 0)
 				break;
 
-#if 0
-			if (verbose && logtick && ticker == 1)
-				log_message(LOG_DEBUG, "ping select timeout = %2ld.%06ld seconds",
-				       dtimeout.tv_sec, dtimeout.tv_usec);
-#endif
-
-			if (select(sock_fp + 1, &fdmask, NULL, NULL, &dtimeout) >= 1) {
-				/* read reply */
-				fromlen = sizeof(from);
-				if (recvfrom(sock_fp, packet, PKBUF_SIZE, 0, (struct sockaddr *)&from, &fromlen) < 0) {
-					int err = errno;
-
-					if (err != EINTR) {
-						log_message(LOG_ERR, "recvfrom gave errno = %d = '%s'", err, strerror(err));
-						return (err);
-					}
-				} else {
-					/* check if packet is our ECHO */
-					icp = (struct icmphdr *)(packet + (((struct ip *)packet)->ip_hl << 2));
-
-					if (icp->type == ICMP_ECHOREPLY) {
-						int rcv_id  = ntohs(icp->un.echo.id);
-						int rcv_seq = ntohs(icp->un.echo.sequence);
-
-						/* Have ping reply, but is it the one we just sent? */
-						if (rcv_id  == daemon_pid &&
-							rcv_seq == (i + 1) &&
-							from.sin_addr.s_addr == to_in->sin_addr.s_addr) {
-
-							if (verbose && logtick && ticker == 1) {
-								/* Report time since tstart in milliseconds (like 'ping' program). */
-								double msec;
-								gettimeofday(&dtimeout, NULL);
-								timersub(&dtimeout, &tstart, &dtimeout);
-								msec = 1.0e3 * (dtimeout.tv_sec + 1.0e-6 * dtimeout.tv_usec);
-								log_message(LOG_DEBUG, "got answer on ping=%d from target %-15s time=%.3fms", i+1, target, msec);
-							}
-
-							return (ENOERR);
-						}
-					}
+			if (found_ping(packet, sock_fp, to, i, &err, &fdmask, &dtimeout)) {
+				/* If successful and verbose, report this. */
+				if (err == 0 && verbose && logtick && ticker == 1) {
+					/* Report time since tstart in milliseconds (like 'ping' program). */
+					double msec;
+					clock_gettime(CLOCK_MONOTONIC, &dtimeout);
+					timespecsub(&dtimeout, &tstart, &dtimeout);
+					msec = 1.0e3 * (dtimeout.tv_sec + 1.0e-9 * dtimeout.tv_nsec);
+					log_message(LOG_DEBUG, "got answer on ping=%d from target %-15s time=%.3fms", i+1, target, msec);
 				}
+				return err;
 			}
 		}
 	}
@@ -185,6 +214,29 @@ int check_net(char *target, int sock_fp, struct sockaddr to, unsigned char *pack
 	return (ENETUNREACH);
 }
 
+/*
+ * Close socket and free the packet buffer. As we zero this memory when originally
+ * allocating it, a non-NULL packet buffer is an indicator it was opened.
+ */
+
+static int close_net(struct pingmode *net)
+{
+	int err = ENOERR;
+
+	if (net->packet != NULL) {
+		free(net->packet);
+		net->packet = NULL;
+
+		if (close(net->sock_fp) < 0) {
+			err = errno;
+			log_message(LOG_ERR, "error closing socket (err = %d = '%s')", err, strerror(err));
+		}
+		net->sock_fp = -1;
+	}
+
+	return err;
+}
+
 /*
  * Set up pinging if in ping mode
  */
@@ -211,6 +263,8 @@ int open_netcheck(struct list *tlist)
 			struct pingmode *net = &act->parameter.net; /* 'net' is alias of act->parameter.net */
 			struct sockaddr_in *to_in;
 
+			close_net(net);
+
 			/* setup the socket */
 			memset(&(net->to), 0, sizeof(struct sockaddr));
 			/*
@@ -259,3 +313,21 @@ int open_netcheck(struct list *tlist)
 
 	return 0;
 }
+
+/*
+ * Shut sockets and free memory as allocated by open_netcheck().
+ */
+
+int close_netcheck(struct list *tlist)
+{
+	int err = 0;
+	struct list *act;
+
+	if (tlist != NULL) {
+		for (act = tlist; act != NULL; act = act->next) {
+			err |= close_net(&act->parameter.net);
+		}
+	}
+
+	return err;
+}
