@@ -3,7 +3,7 @@
  *              Michael Meskes
  * License:	GPL
  *
- * Filename:    $Id: wd_keepalive.c,v 1.6 2007/08/17 09:24:54 meskes Exp $    
+ * Filename:    $Id: wd_keepalive.c,v 1.6 2007/08/17 09:24:54 meskes Exp $
  * Author:      Marcel Jansen, 22 February 2001
  * 		Michael Meskes, since then
  * Purpose:     This program can be run during critical periods
@@ -39,29 +39,20 @@
 #define TRUE  1
 #define FALSE 0
 
-volatile sig_atomic_t _running = 1;
-
 static void usage(char *progname)
 {
 	fprintf(stderr, "%s version %d.%d, usage:\n", progname, MAJOR_VERSION, MINOR_VERSION);
-	fprintf(stderr, "%s [-c | --config-file <config_file>]\n", progname);
+	fprintf(stderr, "%s [options]\n", progname);
+	fprintf(stderr, "options:\n");
+	fprintf(stderr, "  -c | --config-file <file>  specify location of config file\n");
+	fprintf(stderr, "  -X | --loop-exit <number>  run a fixed number of loops then exit\n");
 	exit(1);
 }
 
-/* write a log entry on exit */
-static void log_end(void)
-{
-	/* Log the closing message */
-	log_message(LOG_INFO, "stopping watchdog keepalive daemon (%d.%d)", MAJOR_VERSION, MINOR_VERSION);
-	close_logging();
-	usleep(100000);		/* 0.1s to make sure log is written */
-	return;
-}
-
 /* Dummy function for keep_alive.c use */
 int write_heartbeat(void)
 {
-return 0;
+	return 0;
 }
 
 /* close the device and check for error */
@@ -70,31 +61,29 @@ static void close_all(void)
 	close_watchdog();
 }
 
-void sigterm_handler(int arg)
-{
-	_running = 0;
-}
-
 /* on exit we close the device and log that we stop */
-void terminate(void)
+void terminate(int ecode)
 {
+	log_message(LOG_NOTICE, "stopping watchdog keepalive daemon (%d.%d)", MAJOR_VERSION, MINOR_VERSION);
 	unlock_our_memory();
 	close_all();
 	remove_pid_file();
-	log_end();
-	exit(0);
+	close_logging();
+	xusleep(100000);		/* 0.1s to make sure log is written */
+	exit(ecode);
 }
 
 int main(int argc, char *const argv[])
 {
 	char *configfile = CONFIG_FILENAME;
-	pid_t child_pid;
-	int count = 0;
+	long count = 0L;
+	long count_max = 0L;
 	int c;
 	char *progname;
+	unsigned long twait;
 
 	/* allow all options watchdog understands too */
-	char *opts = "d:i:n:fsvbql:p:t:c:r:m:a:";
+	char *opts = "d:i:n:fsvbql:p:t:c:r:m:a:X:";
 	struct option long_options[] = {
 		{"config-file", required_argument, NULL, 'c'},
 		{"force", no_argument, NULL, 'f'},
@@ -102,6 +91,7 @@ int main(int argc, char *const argv[])
 		{"no-action", no_argument, NULL, 'q'},
 		{"verbose", no_argument, NULL, 'v'},
 		{"softboot", no_argument, NULL, 'b'},
+		{"loop-exit", required_argument, NULL, 'X'},
 		{NULL, 0, NULL, 0}
 	};
 
@@ -129,6 +119,11 @@ int main(int argc, char *const argv[])
 		case 'q':
 		case 'v':
 			break;
+		case 'X':
+			count_max = atol(optarg);
+			log_message(LOG_WARNING, "NOTE: Using --loop-exit so daemon will exit after %ld time intervals",
+				    count_max);
+			break;
 		default:
 			usage(progname);
 		}
@@ -136,46 +131,16 @@ int main(int argc, char *const argv[])
 
 	read_config(configfile);
 
-	/* make sure we're on the root partition */
-	if (chdir("/") < 0) {
-		perror(progname);
-		exit(1);
-	}
-#if !defined(DEBUG)
-	/* fork to go into the background */
-	if ((child_pid = fork()) < 0) {
-		perror(progname);
-		exit(1);
-	} else if (child_pid > 0) {
-		/* fork was okay          */
-		/* wait for child to exit */
-		if (waitpid(child_pid, NULL, 0) != child_pid) {
-			perror(progname);
-			exit(1);
-		}
-		/* and exit myself */
+	/* this daemon has no other function than writing to this device
+	 * i.e. if there is no device given we better punt */
+	if (devname == NULL) {
+		log_message(LOG_INFO, " no watchdog device configured, aborting");
 		exit(0);
 	}
-	/* and fork again to make sure we inherit all rights from init */
-	if ((child_pid = fork()) < 0) {
-		perror(progname);
-		exit(1);
-	} else if (child_pid > 0)
-		exit(0);
-#endif				/* !DEBUG */
 
-	/* now we're free */
-
-#if !defined(DEBUG)
-	/* Okay, we're a daemon     */
-	/* but we're still attached to the tty */
-	/* create our own session */
-	setsid();
-	/* with USE_SYSLOG we don't do any console IO */
-	close(0);
-	close(1);
-	close(2);
-#endif				/* !DEBUG */
+	if (wd_daemon(0, 0)) {
+		fatal_error(EX_SYSERR, "failed to daemonize (%s)", strerror(errno));
+	}
 
 	/* tuck my process id away */
 	if (write_pid_file(KA_PIDFILE)) {
@@ -184,22 +149,12 @@ int main(int argc, char *const argv[])
 
 	/* Log the starting message */
 	open_logging(NULL, MSG_TO_SYSLOG);
-	log_message(LOG_INFO, "starting watchdog keepalive daemon (%d.%d):", MAJOR_VERSION, MINOR_VERSION);
-	if (devname == NULL)
-		log_message(LOG_INFO, " no watchdog device configured, aborting");
-	else
-		log_message(LOG_INFO, " int=%d alive=%s realtime=%s", tint, devname, realtime ? "yes" : "no");
-
-	/* this daemon has no other function than writing to this device 
-	 * i.e. if there is no device given we better punt */
-	if (devname == NULL)
-		terminate();
+	log_message(LOG_NOTICE, "starting watchdog keepalive daemon (%d.%d):", MAJOR_VERSION, MINOR_VERSION);
+	log_message(LOG_INFO, " int=%d alive=%s realtime=%s", tint, devname, realtime ? "yes" : "no");
 
 	/* open the device */
 	if (open_watchdog(devname, dev_timeout) < 0) {
-		remove_pid_file();
-		log_end();
-		exit(1);
+		terminate(1);
 	}
 
 	/* set signal term to call sigterm_handler() */
@@ -208,16 +163,23 @@ int main(int argc, char *const argv[])
 
 	lock_our_memory(realtime, schedprio, daemon_pid);
 
+	twait = (tint * 1000000);
+
 	/* main loop: update after <tint> seconds */
 	while (_running) {
 		keep_alive();
 		/* finally sleep some seconds */
-		sleep(tint);
+		xusleep(twait);
 
 		count++;
+
+		if (count_max > 0 && count >= count_max) {
+			log_message(LOG_WARNING, "loop exit on interval counter reached");
+			_running = 0;
+		}
 	}
 
-	terminate();
+	terminate(0);
 	/* not reached */
-return 0;
+	return 0;
 }
